package cn.zhangdeman.filter;


import cn.zhangdeman.Context;
import cn.zhangdeman.Ordered;
import cn.zhangdeman.Request;
import jakarta.servlet.FilterChain;
import jakarta.servlet.FilterConfig;
import jakarta.servlet.ServletException;
import jakarta.servlet.annotation.WebFilter;
import jakarta.servlet.http.Cookie;
import jakarta.servlet.http.HttpFilter;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import org.springframework.core.annotation.Order;

import java.util.Enumeration;
import java.util.HashMap;
import java.util.Map;


// 初始化请求
@WebFilter("/*") // 所有接口请求均会触发次过滤器
@Order(Ordered.REQUEST_ID) // 足够小, 保证最先执行
public class RequestInitFilter extends HttpFilter {
    @Override
    public void init(FilterConfig filterConfig) throws ServletException {
    }

    @Override
    public void destroy() {
    }

    @Override
    public void doFilter(HttpServletRequest httpServletRequest, HttpServletResponse servletResponse, FilterChain filterChain) {
        Context context = new Context();
        setBaseInfo(httpServletRequest, context); // 设置基础信息
        setRequestQuery(context.getRequestInfo()); // 设置请求query信息
        setRequestBody(context.getRequestInfo()); // 设置请求Body
        setRequestHeaderAndCookie(context.getRequestInfo()); // 填充请求信息: header + cookie
        setRequestId(context); // 设置请求ID, 每次请求需要重新生成
    }

    // 设置基础信息
    private void setBaseInfo(HttpServletRequest httpServletRequest, Context context) {
        // request 信息
        Request request = new Request();
        request.setHttpServletRequest(httpServletRequest);
        request.setClientIp(httpServletRequest.getRemoteAddr()); // client ip
        request.setRequestMethod(httpServletRequest.getMethod()); // 请求类型
        request.setRequestContentType(httpServletRequest.getContentType()); // 请求类型
        request.setRequestUri(httpServletRequest.getRequestURI()); // 请求uri
        // server 信息
        context.setServerIp(httpServletRequest.getLocalAddr()); // 服务器IP
        context.setServerHostname(httpServletRequest.getLocalName()); // 服务器名称
        context.setRequestInfo(request); // 请求信息
        context.setStartTimeStamp(System.currentTimeMillis()); // 开始请求时间
    }

    // 设置请求Query
    private void setRequestQuery(Request request) {
        Map<String, String> query = new HashMap<>();
        HttpServletRequest httpServletRequest = request.getHttpServletRequest(); // 原始请求实例
        Map<String, String[]> queryParamList = httpServletRequest.getParameterMap(); // query参数列表
        for (Map.Entry<String, String[]> entity : queryParamList.entrySet()) {
            if (entity.getValue().length == 0) {
                // 参数值为空
                query.put(entity.getKey(), "");
            } else {
                // 多个参数值,  取第一个
                query.put(entity.getKey(), entity.getValue()[0]);
            }
        }
        request.setRequestQuery(query);
    }

    // 设置请求Body
    private void setRequestBody(Request request) {
        HttpServletRequest httpServletRequest = request.getHttpServletRequest(); // 原始请求实例
        // httpServletRequest.getInputStream()
    }


    // 设置请求信息: header + cookie
    private void setRequestHeaderAndCookie(Request request) {
        HttpServletRequest httpServletRequest = request.getHttpServletRequest(); // 原始请求实例

        request.setRequestHeader(new HashMap<>()); // 请求header
        request.setRequestCookie(new HashMap<>()); // 请求cookie
        // 全部请求header
        Enumeration<String> enumeration = httpServletRequest.getHeaderNames();
        while (enumeration.hasMoreElements()) {
            String name = enumeration.nextElement();
            if (name.equalsIgnoreCase("cookie")) {
                // cookie单独摘出来处理
                continue;
            }
            String value = httpServletRequest.getHeader(name);
            request.getRequestHeader().put(name, value);
        }
        // 全部请求cookie
        Cookie[] cookieList = httpServletRequest.getCookies();
        for (Cookie cookie : cookieList) {
            request.getRequestCookie().put(cookie.getName(), cookie.getValue());
        }
    }

    // 设置请求ID
    private void setRequestId(Context context) {
    }
}
